@@ -85,13 +85,19 @@ class QueryHelper:
                 filters_to_apply.append(field_expr)
         return filters_to_apply
 
+    @classmethod
+    def get_path_root(cls, path):
+        if path.prev is None:
+            return path.key
+        return cls.get_path_root(path.prev)
+
     @classmethod
     def parse_query(cls, info) -> List[QueryField]:
-        # TODO: cache was removed because of bug with two diff. fragments
-        #  with the same ObjectType and different fields in each fragment
-        # if not hasattr(
-        #     info.context, "parsed_query"
-        # ) or not info.context.parsed_query.get(info.field_name):
+        path_root = cls.get_path_root(info.path)
+        if hasattr(info.context, "parsed_query") and info.context.parsed_query.get(
+            path_root
+        ):
+            return info.context.parsed_query[path_root]
 
         nodes = info.field_nodes
 
@@ -103,19 +109,11 @@ class QueryHelper:
         fragments = cls.__parse_fragments(info.fragments, variables)
 
         result = cls.__set_fragment_fields(result, fragments)
+        setattr(info.context, "parsed_query", {path_root: result})
         return result
-        # setattr(info.context, "parsed_query", {info.field_name: result})
-
-        # return info.context.parsed_query[info.field_name].copy()
 
     @classmethod
-    def get_selected_fields(cls, info, model, sort=None):
-        object_types = getattr(info.context, "object_types", {})
-        object_type = object_types.get(info.field_name)
-
-        if not object_type:
-            return
-
+    def get_selected_fields(cls, info, model, object_type, sort=None):
         gql_field = cls.get_current_field(info)
 
         object_type_fields = {}
@@ -131,8 +129,7 @@ class QueryHelper:
                 else:
                     object_type_fields[_name] = attr
 
-        type_ = info.context.object_types[info.field_name]
-        meta_fields = type_._meta.fields
+        meta_fields = object_type._meta.fields
 
         select_fields = set()
         if isinstance(model, Table):
@@ -189,7 +186,7 @@ class QueryHelper:
 
             model_field = getattr(current_field, "model_field", None)
             if model_field is not None:
-                # labled columns could create name conflict
+                # labeled columns could create name conflict
                 if (
                     getattr(current_field, "use_label", True)
                     and field != model_field.key
@@ -203,15 +200,34 @@ class QueryHelper:
     @classmethod
     def get_current_field(cls, info) -> Optional[QueryField]:
         gql_fields = cls.parse_query(info)
-        field_name = camel_to_snake(info.field_name)
 
-        while gql_fields:
-            next_field = gql_fields.pop()
-            if next_field.name == field_name:
-                return next_field
+        def __get_prev_field(path, current_field):
+            if path.prev is None:
+                if path.typename == "Mutation":
+                    return current_field
+                return current_field[0]
 
-            if next_field.values:
-                gql_fields.extend(next_field.values)
+            prev_field = __get_prev_field(path.prev, current_field)
+
+            if path.typename is None or path.key in ["edges", "node"]:
+                return prev_field
+            path_name = camel_to_snake(path.key)
+            if isinstance(prev_field, list):
+                fields = prev_field
+            else:
+                if prev_field.name == path_name:
+                    return prev_field
+
+                fields = prev_field.values
+            for field in fields:
+                if field.name == path_name:
+                    return field
+
+            return prev_field
+
+        result = __get_prev_field(info.path, gql_fields)
+
+        return result
 
     @classmethod
     def has_arg(cls, info, arg: str):
@@ -297,24 +313,40 @@ class QueryHelper:
 
     @classmethod
     def __set_fragment_fields(cls, parsed_query, fragments) -> list:
-        new_values = []
+        new_values = {}
+
+        def _proc_fragment(field_):
+            extra_fields_ = fragments.get(field_.name)
+            for extra_field_ in extra_fields_:
+                if isinstance(extra_field_, FragmentField):
+                    _proc_fragment(extra_field_)
+                elif extra_field_.values:
+                    extra_field_.values = cls.__set_fragment_fields(
+                        parsed_query=extra_field_.values, fragments=fragments
+                    )
+                existing_field: QueryField
+                if existing_field := new_values.get(extra_field_.name):
+                    if existing_field.values is None and extra_field_.values is None:
+                        continue
+                    existing_field.values.extend(extra_field_.values)
+                else:
+                    new_values[extra_field_.name] = extra_field_
+
+        fragment_fields = []
         for field in parsed_query:
             if isinstance(field, FragmentField):
-                extra_fields = fragments.get(field.name)
-                for extra_field in extra_fields:
-                    if extra_field.values:
-                        extra_field.values = cls.__set_fragment_fields(
-                            parsed_query=extra_field.values, fragments=fragments
-                        )
-                new_values.extend(extra_fields)
+                fragment_fields.append(field)
             else:
                 if field.values:
                     field.values = cls.__set_fragment_fields(
                         parsed_query=field.values, fragments=fragments
                     )
 
-                new_values.append(field)
-        return new_values
+                new_values[field.name] = field
+        for field in fragment_fields:
+            _proc_fragment(field)
+
+        return list(new_values.values())
 
     @classmethod
     def __get_query_field_page_info_fields(cls, field) -> Set[str]:
